#include <math.h>
#include <stdio.h>
#include "fixglo.h"
#include "lssch.h"
#include "matrix.h"

int lssch (kvektor x[], double y[], long n1)
/* Berechnung des LS-Schaetzers (0, falls xtx nicht invertierbar) */
/* Benoetigt globale Variable p */
/* Veraendert globale Variable th0, r */
/* Benoetigt Funktionen schmult, minv */
/* Benoetigt NKOR, PKOR */
{
  long i, j;
  double xtw[PKOR-1][NKOR];

  for (i=0; i<p; ++i)
  {
    for (j=0; j<n1; ++j)
      xtw[i][j]=x[j][i];
  }
  return(schmult (x, y, xtw, n1));
}

double lsscale(kvektor x1[], double y1[], vektor beta, long n1)
/* Residuen und Skalenschaetzer fuer Regressionsschaetzer beta */
/* Benoetigt globale Variable p */
/* Veraendert globale Variable r */
/* Benoetigt Funktionen resid, vresid */
{
  double su;
  long i;

  resid (x1, y1, beta, n1);
  su=0;
  for (i=0; i<n1; ++i)
	 su+=r[i];
  return(su/(n1-p-1));
}

int schmult(kvektor x1[], double y1[], double xtw[][NKOR], long n1)
/* Zur Berechnung von Regressionsschaetzern noetige Matrixmultiplikationen */
/* Benoetigt globale Variable p */
/* Veraendert globale Variable th0 */
/* Benoetigt Funktion minv */
/* Benoetigt PKOR */
{
  long z, z1, z2;
  int invcon;
  vektor vp;
  vektor xtxi[PKOR];

  for (z=0; z<=p; ++z)
	 for (z1=0; z1<=p; ++z1)
	 {
		if (z1<z)
		  xtxi[z][z1]=xtxi[z1][z];
		else
		{
			xtxi[z][z1]=0;
			for (z2=0; z2<n1; ++z2)
			{
			 if ((z!=p) && (z1!=p))
				xtxi[z][z1]+=xtw[z][z2]*x1[z2][z1];
			 if ((z!=p) && (z1==p))
				xtxi[z][z1]+=xtw[z][z2];
			 if ((z==p) && (z1==p))
				xtxi[z][z1]+=1;
			} /* for z2*/
		} /* else */
  } /* for z1 */
  for (z=0; z<p; ++z)
  {
	 vp[z]=0;
	 for (z2=0; z2<n1; ++z2)
		vp[z]+=xtw[z][z2]*y1[z2];
  }
  vp[p]=0;
  for (z2=0; z2<n1; ++z2)
	 vp[p]+=y1[z2];
  invcon=minv (xtxi, p+1);
  if (invcon)
  {
	 for (z=0; z<p; ++z)
    {
      th0[z]=0;
      for (z2=0; z2<p; ++z2)
        th0[z]+=xtxi[z][z2]*vp[z2];
      th0[z]+=xtxi[z][p]*vp[p];
    }
    th0[p]=0;
    for (z2=0; z2<p; ++z2)
      th0[p]+=xtxi[p][z2]*vp[z2];
    th0[p]+=xtxi[p][p]*vp[p];
  }
  return(invcon);
}

void resid(kvektor x[], double y[], vektor th0, long n)
/* Berechnung der quadratischen Residuen */
/* Benoetigt globale Variable p */
/* Veraendert globale Variable r */
/* Benoetigt Funktion vresid */
{
  long j;

  vresid (x, y, th0, n);
  for (j=0; j<n; ++j)
    r[j]=pow(r[j],2);
}

void vresid(kvektor x[], double y[], double th0[], long n)
/* Berechnung der Residuen mit Vorzeichen */
/* Benoetigt globale Variable p */
/* Veraendert globale Variable r */
{
  double yd;
  long i,j;

  for (j=0; j<n; ++j)
  {
    yd=0;
    for (i=0; i<p; ++i)
      yd+=th0[i]*x[j][i];
    yd+=th0[p];
    r[j]=y[j]-yd;
  }
}
