#include <math.h>
#include <stdio.h>
#include <string.h>
#include "fixglo.h"
#include "regstan.h"

void standard(void)
/* Standardisiert Matrix x und Vektor y mit Median und Dispersion */
/* Benoetigt globale Variable p, n */
/* Veraendert globale Variable x, y, mx, my, dx, dy */
/* Benoetigt Funktionen median, disp */
/* Benoetigt NKOR, PKOR */
{
  long i,j;
  double xtw[PKOR][NKOR];

  for (i=0; i<n; ++i)
	for (j=0; j<p; ++j)
	  xtw[j][i]=x[i][j];
  my=median(y,n);
  for (i=0; i<p; ++i)
  {
	mx[i]=median(xtw[i],n);
  }
  for (i=0; i<n; ++i)
	{
	  y[i]=y[i]-my;
	  for (j=0; j<p; ++j)
		xtw[j][i]=xtw[j][i]-mx[j];
	}
  dy=disp(y,n);
  for (i=0; i<p; ++i)
	dx[i]=disp(xtw[i],n);
  for (i=0; i<n; ++i)
	{
          if (dy>1e-8)
	    y[i]=y[i]/dy;
	  for (j=0; j<p; ++j)
          {
            if(dx[j]>1e-8)
		xtw[j][i]=xtw[j][i]/dx[j];
          }
	}
  for (i=0; i<n; ++i)
	for (j=0; j<p; ++j)
	  x[i][j]=xtw[j][i];
}

void restandard(vektor th0)
/* Rechnet den Vektor th0 in vorstandardisierten Zustand zurueck */
/* Benoetigt globale Variable th0 */
/* Veraendert globale Variable mx, my, dx, dy, p */
{
  long j;
  double su = 0;

  for (j=0; j<p; ++j)
  {
    if (dx[j]<=1e-8)
      dx[j]=1;
    su += th0[j]*mx[j]/dx[j];
    th0[j] *= dy/dx[j];
  }
  if (dy<=1e-8)
      dy=1;
  th0[p] = dy*(th0[p]-su)+my;
}

void stanausgabe(void)
/* Ausgabe der standardisierten Werte auf Ausgabematrix */
/* Benoetigt globale Variable x, y, p, n, mx, my, dx, dy, out, ico, id, varname */
{
  long i,j;

  fprintf(out,"\nStandardization to median=0, median absolute deviation=1");
  fprintf(out,"\nOutput of the standardized values, first dependent, then indep. variable \n");
  fprintf(out,"Median %s = %16.7f MAD = %16.7f \n \n",varname[1],my,dy);
  if (dy<=1e-8)
  { 
    fprintf(out,"MAD too small -> no scale-standardization of this variable.\n");
    dy=1;
  }
  for (i=0; i<p; ++i)
  {
    fprintf(out,"Median %s = %16.7f MAD = %16.7f \n",varname[2+i],mx[i],dx[i]);
    if (dx[i]<=1e-8)
    {
      fprintf(out,"MAD too small -> no scale-standardization of this variable.\n");
      dx[i]=1;
    }
  }
  fprintf(out,"\n");
  if (ico)
    fprintf(out,"%12s ",varname[0]);
  fprintf(out," %s",varname[1]);
  for (i=strlen(varname[1]); i<10; ++i)
    fprintf(out," ");
  for (j=0; j<p; ++j)
  {
    fprintf(out,"%s",varname[j+2]);
    for (i=strlen(varname[j+2]); i<10; ++i)
      fprintf(out," ");
  }
  fprintf(out,"\n");
  for (j=0; j<n; ++j)
  {
    if (ico)
      fprintf(out,"%12s ",id[j]);
    fprintf(out,"%10.6g ",y[j]);
    for (i=0; i<p; ++i)
      fprintf(out,"%10.6g ",x[j][i]);
    fprintf(out,"\n");
  }
  fprintf(out,"\n");
}
